package sun.misc;

import java.util.ArrayList;
import java.util.function.Function;
import java.util.List;
import java.security.AccessController;
import java.security.Security;
import java.io.SerializablePermission;
import java.util.Objects;
import java.io.ObjectInputStream;
import sun.util.logging.PlatformLogger;

@FunctionalInterface
public interface ObjectInputFilter
{
    public abstract ObjectInputFilter.Status checkInput(ObjectInputFilter.FilterInfo p0);
    
    public enum Status
    {
        UNDECIDED, 
        ALLOWED, 
        REJECTED;
    }
    
    public static final class Config
    {
        private static final Object serialFilterLock;
        private static final PlatformLogger configLog;
        private static final String SERIAL_FILTER_PROPNAME = "jdk.serialFilter";
        private static final ObjectInputFilter configuredFilter;
        private static ObjectInputFilter serialFilter;
        
        private Config() {
        }
        
        static void filterLog(PlatformLogger.Level level, String msg, Object... params) {
            if (ObjectInputFilter.Config.configLog != null) {
                if (PlatformLogger.Level.INFO.equals(level)) {
                    ObjectInputFilter.Config.configLog.info(msg, params);
                }
                else if (PlatformLogger.Level.WARNING.equals(level)) {
                    ObjectInputFilter.Config.configLog.warning(msg, params);
                }
                else {
                    ObjectInputFilter.Config.configLog.severe(msg, params);
                }
            }
        }
        
        public static ObjectInputFilter getObjectInputFilter(ObjectInputStream obj) {
            Objects.requireNonNull(obj, "inputStream");
            return SharedSecrets.getJavaOISAccess().getObjectInputFilter(obj);
        }
        
        public static void setObjectInputFilter(ObjectInputStream obj, ObjectInputFilter objectInputFilter) {
            Objects.requireNonNull(obj, "inputStream");
            SharedSecrets.getJavaOISAccess().setObjectInputFilter(obj, objectInputFilter);
        }
        
        public static ObjectInputFilter getSerialFilter() {
            synchronized (ObjectInputFilter.Config.serialFilterLock) {
                return ObjectInputFilter.Config.serialFilter;
            }
        }
        
        public static void setSerialFilter(ObjectInputFilter objectInputFilter) {
            Objects.requireNonNull(objectInputFilter, "filter");
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkPermission(new SerializablePermission("serialFilter"));
            }
            synchronized (ObjectInputFilter.Config.serialFilterLock) {
                if (ObjectInputFilter.Config.serialFilter != null) {
                    throw new IllegalStateException("Serial filter can only be set once");
                }
                ObjectInputFilter.Config.serialFilter = objectInputFilter;
            }
        }
        
        public static ObjectInputFilter createFilter(String obj) {
            Objects.requireNonNull(obj, "pattern");
            return ObjectInputFilter.Config.Global.createFilter(obj, true);
        }
        
        public static ObjectInputFilter createFilter2(String obj) {
            Objects.requireNonNull(obj, "pattern");
            return ObjectInputFilter.Config.Global.createFilter(obj, false);
        }
        
        static {
            serialFilterLock = new Object();
            configuredFilter = (ObjectInputFilter)AccessController.doPrivileged(()/*  */ -> {
                String property = VM.getSavedProperty("jdk.serialFilter");
                if (property == null) {
                    property = Security.getProperty("jdk.serialFilter");
                }
                if (property != null) {
                    PlatformLogger platformLogger = PlatformLogger.getLogger("java.io.serialization");
                    platformLogger.info("Creating serialization filter from {0}", new Object[] { property });
                    try {
                        return createFilter(property);
                    }
                    catch (RuntimeException t) {
                        platformLogger.warning("Error configuring filter: {0}", t);
                    }
                }
                return null;
            });
            configLog = ((ObjectInputFilter.Config.configuredFilter != null) ? PlatformLogger.getLogger("java.io.serialization") : null);
            ObjectInputFilter.Config.serialFilter = ObjectInputFilter.Config.configuredFilter;
        }
        
        static final class Global implements ObjectInputFilter
        {
            private final String pattern;
            private final List<Function<Class<?>, ObjectInputFilter.Status>> filters;
            private long maxStreamBytes;
            private long maxDepth;
            private long maxReferences;
            private long maxArrayLength;
            private final boolean checkComponentType;
            
            static ObjectInputFilter createFilter(String s, boolean b) {
                ObjectInputFilter.Config.Global global = new ObjectInputFilter.Config.Global(s, b);
                return global.isEmpty() ? null : global;
            }
            
            private Global(String str, boolean checkComponentType) {
                this.pattern = str;
                this.checkComponentType = checkComponentType;
                this.maxArrayLength = Long.MAX_VALUE;
                this.maxDepth = Long.MAX_VALUE;
                this.maxReferences = Long.MAX_VALUE;
                this.maxStreamBytes = Long.MAX_VALUE;
                String[] split = str.split(";");
                this.filters = new ArrayList(split.length);
                for (int i = 0; i < split.length; ++i) {
                    String s = split[i];
                    int length = s.length();
                    if (length != 0 && !this.parseLimit(s)) {
                        int n = s.charAt(0) == '!';
                        if (s.indexOf(47) >= 0) {
                            throw new IllegalArgumentException("invalid character \"/\" in: \"" + str + "\"");
                        }
                        if (s.endsWith("*")) {
                            if (s.endsWith(".*")) {
                                if (s.substring((int)n, length - 1).length() < 2) {
                                    throw new IllegalArgumentException("package missing in: \"" + str + "\"");
                                }
                                if (!(n == 0)) {
                                    this.filters.add(clazz8/* java.lang.Class, */ -> matchesPackage(clazz8, var_8_CC) ? ObjectInputFilter.Status.REJECTED : ObjectInputFilter.Status.UNDECIDED);
                                }
                                else {
                                    this.filters.add(clazz7/* java.lang.Class, */ -> matchesPackage(clazz7, var_8_CC) ? ObjectInputFilter.Status.ALLOWED : ObjectInputFilter.Status.UNDECIDED);
                                }
                            }
                            else if (s.endsWith(".**")) {
                                if (s.substring((int)n, length - 2).length() < 2) {
                                    throw new IllegalArgumentException("package missing in: \"" + str + "\"");
                                }
                                if (!(n == 0)) {
                                    this.filters.add(clazz6/* java.lang.Class, */ -> clazz6.getName().startsWith(var_8_141) ? ObjectInputFilter.Status.REJECTED : ObjectInputFilter.Status.UNDECIDED);
                                }
                                else {
                                    this.filters.add(clazz5/* java.lang.Class, */ -> clazz5.getName().startsWith(var_8_141) ? ObjectInputFilter.Status.ALLOWED : ObjectInputFilter.Status.UNDECIDED);
                                }
                            }
                            else {
                                s.substring((int)n, length - 1);
                                if (!(n == 0)) {
                                    this.filters.add(clazz4/* java.lang.Class, */ -> clazz4.getName().startsWith(var_8_1AC) ? ObjectInputFilter.Status.REJECTED : ObjectInputFilter.Status.UNDECIDED);
                                }
                                else {
                                    this.filters.add(clazz3/* java.lang.Class, */ -> clazz3.getName().startsWith(var_8_1AC) ? ObjectInputFilter.Status.ALLOWED : ObjectInputFilter.Status.UNDECIDED);
                                }
                            }
                        }
                        else {
                            if (s.substring((int)n).isEmpty()) {
                                throw new IllegalArgumentException("class or package missing in: \"" + str + "\"");
                            }
                            if (!(n == 0)) {
                                this.filters.add(clazz2/* java.lang.Class, */ -> clazz2.getName().equals(var_8_1EA) ? ObjectInputFilter.Status.REJECTED : ObjectInputFilter.Status.UNDECIDED);
                            }
                            else {
                                this.filters.add(clazz/* java.lang.Class, */ -> clazz.getName().equals(var_8_1EA) ? ObjectInputFilter.Status.ALLOWED : ObjectInputFilter.Status.UNDECIDED);
                            }
                        }
                    }
                }
            }
            
            private boolean isEmpty() {
                return this.filters.isEmpty() && this.maxArrayLength == Long.MAX_VALUE && this.maxDepth == Long.MAX_VALUE && this.maxReferences == Long.MAX_VALUE && this.maxStreamBytes == Long.MAX_VALUE;
            }
            
            private boolean parseLimit(String s) {
                int index = s.indexOf(61);
                if (index < 0) {
                    return false;
                }
                String substring = s.substring(index + 1);
                if (s.startsWith("maxdepth=")) {
                    this.maxDepth = parseValue(substring);
                }
                else if (s.startsWith("maxarray=")) {
                    this.maxArrayLength = parseValue(substring);
                }
                else if (s.startsWith("maxrefs=")) {
                    this.maxReferences = parseValue(substring);
                }
                else if (s.startsWith("maxbytes=")) {
                    this.maxStreamBytes = parseValue(substring);
                }
                else {
                    throw new IllegalArgumentException("unknown limit: " + s.substring(0, index));
                }
                return true;
            }
            
            private static long parseValue(String s) throws IllegalArgumentException {
                long long1 = Long.parseLong(s);
                if (long1 < 0L) {
                    throw new IllegalArgumentException("negative limit: " + s);
                }
                return long1;
            }
            
            @Override
            public ObjectInputFilter.Status checkInput(ObjectInputFilter.FilterInfo filterInfo) {
                if (filterInfo.references() < 0L || filterInfo.depth() < 0L || filterInfo.streamBytes() < 0L || filterInfo.references() > this.maxReferences || filterInfo.depth() > this.maxDepth || filterInfo.streamBytes() > this.maxStreamBytes) {
                    return ObjectInputFilter.Status.REJECTED;
                }
                Class<Object> clazz = filterInfo.serialClass();
                if (clazz != null) {
                    if (clazz.isArray()) {
                        if (filterInfo.arrayLength() >= 0L && filterInfo.arrayLength() > this.maxArrayLength) {
                            return ObjectInputFilter.Status.REJECTED;
                        }
                        if (!this.checkComponentType) {
                            return ObjectInputFilter.Status.UNDECIDED;
                        }
                        do {
                            clazz = clazz.getComponentType();
                        } while (clazz.isArray());
                    }
                    if (clazz.isPrimitive()) {
                        return ObjectInputFilter.Status.UNDECIDED;
                    }
                    return (ObjectInputFilter.Status)this.filters.stream().map(function/* java.util.function.Function, */ -> (ObjectInputFilter.Status)function.apply(var_2_55)).filter(status/* sun.misc.ObjectInputFilter.Status, */ -> status != ObjectInputFilter.Status.UNDECIDED).findFirst().orElse(ObjectInputFilter.Status.UNDECIDED);
                }
                else {
                    return ObjectInputFilter.Status.UNDECIDED;
                }
            }
            
            private static boolean matchesPackage(Class<?> clazz, String prefix) {
                String name = clazz.getName();
                return name.startsWith(prefix) && name.lastIndexOf(46) == prefix.length() - 1;
            }
            
            @Override
            public String toString() {
                return this.pattern;
            }
        }
    }
    
    public interface FilterInfo
    {
        public abstract Class<?> serialClass();
        
        public abstract long arrayLength();
        
        public abstract long depth();
        
        public abstract long references();
        
        public abstract long streamBytes();
    }
}
